#pragma once
#include "scalar.h"

/* a class for vectors */
class Vector {
public:
    Scalar x, y, z; // the Cartesian coords

    /* constructors */
    Vector() : x(0), y(0), z(0) {}
    Vector(Scalar a, Scalar b, Scalar c) : x(a), y(b), z(c) {}

    /* declaration of methods */
    Vector sum(const Vector& other) const;
    Vector operator*(Scalar k) const {
        return Vector(x * k, y * k, z * k);
    }


    // another way to write down the vector sum
    Vector operator+(const Vector& other) const;
    Vector operator-(const Vector& other) const {
        return Vector(x - other.x, y - other.y, z - other.z);
    }
    Vector operator-() const {
        return Vector(-x, -y, -z);
    }
};


/* implementation of methods */
inline Vector Vector::sum(const Vector& other) const {
    return Vector(x + other.x, y + other.y, z + other.z);
}

inline Vector Vector::operator+(const Vector& other) const {
    return Vector(x + other.x, y + other.y, z + other.z);
}

/* global functions on vectors and scalars */

inline Scalar dot(const Vector& a, const Vector& b) {
    return (a.x * b.x) + (a.y * b.y) + (a.z * b.z);
}

inline Vector cross(const Vector& a, const Vector& b) {
    return Vector(
        a.y * b.z - a.z * b.y,
        a.z * b.x - a.x * b.z,
        a.x * b.y - a.y * b.x
    ); 
}

inline bool isZero(Vector v) {
    return dot(v, v) < 0.001;
}

inline bool isEqual(Vector v1, Vector v2) {
    return isEqual(v1.x, v2.x) && isEqual(v1.y, v2.y) && isEqual(v1.z, v2.z);
}

/* this is necessary to make expressions like  SCALAR * VECTOR possible (and not only viceversa) */
inline Vector operator*(Scalar k, Vector v) {
    return v * k;
}